# -*- coding: utf-8 -*-
"""
Integration of the pint library to provide physical units.

See pint documentation: http://pint.readthedocs.org/en/latest/
"""
from odin.contrib.pint import units  # noqa
from odin.contrib.pint.fields import *  # noqa



# -*- coding: utf-8 -*-
import unittest
from odin import exceptions


class ValidationException(unittest.TestCase):
    def test_with_string(self):
        test_message = "Test message"
        target = exceptions.ValidationError(test_message)

        self.assertListEqual([test_message], target.messages)
        self.assertFalse(hasattr(target, 'message_dict'))
        self.assertEqual("['Test message']", str(target))
        self.assertEqual("ValidationError(['Test message'])", repr(target))

    def test_with_list(self):
        test_message_list = ["Test message", "Test message 2"]
        target = exceptions.ValidationError(test_message_list)

        self.assertListEqual(test_message_list, target.messages)
        self.assertFalse(hasattr(target, 'message_dict'))
        self.assertEqual("['Test message', 'Test message 2']", str(target))
        self.assertEqual("ValidationError(['Test message', 'Test message 2'])", repr(target))

    def test_with_dict(self):
        test_message_dict = {
            "Test Key 1": ["Test Message 1"],
            "Test Key 2": ["Test Message 2"],
        }
        target = exceptions.ValidationError(test_message_dict)

        self.assertDictEqual(test_message_dict, target.message_dict)
        self.assertListEqual([{
            "Test Key 1": ["Test Message 1"],
            "Test Key 2": ["Test Message 2"],
        }], target.messages)

        expected = {'Test Key 2': ['Test Message 2'], 'Test Key 1': ['Test Message 1']}
        self.assertEqual(expected, target.message_dict)
        self.assertEqual(
            "ValidationError({'Test Key 1': ['Test Message 1'], 'Test Key 2': ['Test Message 2']})",
            repr(target)
        )



# -*- coding: utf-8 -*-
import six
from .exceptions import InvalidPathError
from .traversal import TraversalPath


class FilterAtom(object):
    """
    Base filter statement
    """
    def __name__(self):
        return self.__class__.__name__

    def __call__(self, resource):
        raise NotImplementedError()

    def any(self, collection):
        return any(self(r) for r in collection)

    def all(self, collection):
        return all(self(r) for r in collection)


class FilterChain(FilterAtom):
    operator_name = ''
    check_operator = all

    def __init__(self, *atoms):
        self._atoms = list(atoms)

    def __len__(self):
        return len(self._atoms)

    def __add__(self, other):
        if isinstance(other, self.__class__):
            return self.__class__(*self._atoms + other._atoms)
        elif isinstance(other, FilterComparison):
            self._atoms.append(other)
            return self
        raise TypeError("{} not supported for this operation".format(other))

    def __call__(self, resource):
        return self.check_operator(a(resource) for a in self._atoms)

    def __str__(self):
        if not self._atoms:
            return ''
        pin = " {} ".format(self.operator_name)
        return "({})".format(pin.join(str(a) for a in self._atoms))


class And(FilterChain):
    operator_name = 'AND'
    check_operator = all


class Or(FilterChain):
    operator_name = 'OR'
    check_operator = any


class FilterComparison(FilterAtom):
    """
    Base class for filter operator atoms
    """
    # Symbol for this operator and alternatives. The first item is used when generating
    # a representation of the filter, the others are used for parsing queries.
    operator_symbols = []

    def __init__(self, field, value, operation=None):
        self.field = TraversalPath.parse(field)
        self.value = value
        self.operation = operation

    def __call__(self, resource):
        try:
            value = self.field.get_value(resource)
        except InvalidPathError:
            return False
        else:
            if self.operation:
                value = self.operation(value)
            return self.compare(value)

    def __str__(self):
        value = self.value
        if isinstance(self.value, six.string_types):
            value = "'{}'".format(value)

        if self.operation:
            op_name = getattr(self.operation, 'name', self.operation.__name__)
            return "{}({}) {} {}".format(op_name, self.field, self.operator_symbols[0], value)
        else:
            return "{} {} {}".format(self.field, self.operator_symbols[0], value)

    def compare(self, value):
        raise NotImplementedError()


class Equal(FilterComparison):
    operator_symbols = ('==', '=', 'eq')

    def compare(self, value):
        return value == self.value


class NotEqual(FilterComparison):
    operator_symbols = ('!=', '<>', 'neq')

    def compare(self, value):
        return value != self.value


class LessThan(FilterComparison):
    operator_symbols = ('<', 'lt')

    def compare(self, value):
        return value < self.value


class LessThanOrEqual(FilterComparison):
    operator_symbols = ('<=', 'lte')

    def compare(self, value):
        return value <= self.value


class GreaterThan(FilterComparison):
    operator_symbols = ('>', 'gt')

    def compare(self, value):
        return value > self.value


class GreaterThanOrEqual(FilterComparison):
    operator_symbols = ('>=', 'gte')

    def compare(self, value):
        return value >= self.value



# -*- coding: utf-8 -*-
import unittest
import datetime
import six
from odin import datetimeutil


class TimezonesTestCase(unittest.TestCase):
    def test_utc(self):
        self.assertEqual(datetime.timedelta(0), datetimeutil.utc.utcoffset(datetime.datetime.now()))
        self.assertEqual('UTC', datetimeutil.utc.tzname(datetime.datetime.now()))
        self.assertEqual(datetime.timedelta(0), datetimeutil.utc.dst(datetime.datetime.now()))
        self.assertEqual('UTC', str(datetimeutil.utc))
        self.assertEqual('<timezone: UTC>', repr(datetimeutil.utc))

    def test_fixed_timezone(self):
        target = datetimeutil.FixedTimezone.from_hours_minutes(10, 30)

        self.assertEqual(datetime.timedelta(hours=10, minutes=30), target.utcoffset(datetime.datetime.now()))
        self.assertEqual('10:30', target.tzname(datetime.datetime.now()))
        self.assertEqual(datetime.timedelta(0), target.dst(datetime.datetime.now()))
        self.assertEqual('10:30', str(target))
        self.assertEqual("<timezone '10:30' datetime.timedelta(0, 37800)>", repr(target))

    def test_fixed_timezone_pickling(self):
        import pickle

        target = datetimeutil.FixedTimezone.from_hours_minutes(10, 30)

        data = pickle.dumps(target)
        result = pickle.loads(data)

        self.assertEqual(result, target)


class ParseIsoDateStringTestCase(unittest.TestCase):
    def test_valid_values(self):
        self.assertEqual(
            datetime.date(2014, 1, 13),
            datetimeutil.parse_iso_date_string('2014-01-13')
        )

    def test_invalid_values(self):
        with self.assertRaises(ValueError):
            datetimeutil.parse_iso_date_string(123)
        with self.assertRaises(ValueError):
            datetimeutil.parse_iso_date_string('2014-1-13')
        with self.assertRaises(ValueError):
            datetimeutil.parse_iso_date_string('2014/01/13')


class ParseIsoTimeStringTestCase(unittest.TestCase):
    def test_valid_values(self):
        self.assertEqual(
            datetime.time(23, 53, 25, 0, datetimeutil.FixedTimezone.from_hours_minutes(10, 0)),
            datetimeutil.parse_iso_time_string('23:53:25',
                                               default_timezone=datetimeutil.FixedTimezone.from_hours_minutes(10, 0))
        )
        self.assertEqual(
            datetime.time(23, 53, 25, 0, datetimeutil.utc),
            datetimeutil.parse_iso_time_string('23:53:25')
        )
        self.assertEqual(
            datetime.time(23, 53, 25, 0, datetimeutil.utc),
            datetimeutil.parse_iso_time_string('23:53:25Z')
        )
        self.assertEqual(
            datetime.time(23, 53, 25, 432, datetimeutil.utc),
            datetimeutil.parse_iso_time_string('23:53:25.432Z')
        )
        self.assertEqual(
            datetime.time(23, 53, 25, 0, datetimeutil.FixedTimezone.from_hours_minutes(10, 0)),
            datetimeutil.parse_iso_time_string('23:53:25+10')
        )
        self.assertEqual(
            datetime.time(23, 53, 25, 432, datetimeutil.FixedTimezone.from_hours_minutes(10, 0)),
            datetimeutil.parse_iso_time_string('23:53:25.432+10')
        )
        self.assertEqual(
            datetime.time(23, 53, 25, 0, datetimeutil.FixedTimezone.from_hours_minutes(10, 30)),
            datetimeutil.parse_iso_time_string('23:53:25+10:30')
        )
        self.assertEqual(
            datetime.time(23, 53, 25, 432, datetimeutil.FixedTimezone.from_hours_minutes(-10, -30)),
            datetimeutil.parse_iso_time_string('23:53:25.432-10:30')
        )

    def test_invalid_values(self):
        with self.assertRaises(ValueError):
            datetimeutil.parse_iso_time_string(123)
        with self.assertRaises(ValueError):
            datetimeutil.parse_iso_time_string('23:53:25EST')


class ParseIsoDateTimeStringTestCase(unittest.TestCase):
    def test_valid_values(self):
        self.assertEqual(
            datetime.datetime(2014, 1, 13, 0, 28, 33, 0, datetimeutil.utc),
            datetimeutil.parse_iso_datetime_string('2014-01-13T00:28:33')
        )
        self.assertEqual(
            datetime.datetime(2014, 1, 13, 0, 28, 33, 0, datetimeutil.utc),
            datetimeutil.parse_iso_datetime_string('2014-01-13T00:28:33Z')
        )
        self.assertEqual(
            datetime.datetime(2014, 1, 13, 0, 28, 33, 432, datetimeutil.utc),
            datetimeutil.parse_iso_datetime_string('2014-01-13T00:28:33.432Z')
        )
        self.assertEqual(
            datetime.datetime(2014, 1, 13, 0, 28, 33, 0, datetimeutil.FixedTimezone.from_hours_minutes(10, 0)),
            datetimeutil.parse_iso_datetime_string('2014-01-13T00:28:33+10')
        )
        self.assertEqual(
            datetime.datetime(2014, 1, 13, 0, 28, 33, 432, datetimeutil.FixedTimezone.from_hours_minutes(10, 0)),
            datetimeutil.parse_iso_datetime_string('2014-01-13T00:28:33.432+10')
        )
        self.assertEqual(
            datetime.datetime(2014, 1, 13, 0, 28, 33, 0, datetimeutil.FixedTimezone.from_hours_minutes(10, 30)),
            datetimeutil.parse_iso_datetime_string('2014-01-13T00:28:33+10:30')
        )
        self.assertEqual(
            datetime.datetime(2014, 1, 13, 0, 28, 33, 432, datetimeutil.FixedTimezone.from_hours_minutes(-10, -30)),
            datetimeutil.parse_iso_datetime_string('2014-01-13T00:28:33.432-10:30')
        )
        self.assertEqual(
            datetime.datetime(2014, 1, 13, 0, 28, 33, 432, datetimeutil.FixedTimezone.from_hours_minutes(10, 30)),
            datetimeutil.parse_iso_datetime_string('2014-01-13T00:28:33.432+1030')
        )

        # Non-Standard but close formats that are common
        self.assertEqual(
            datetime.datetime(2014, 1, 13, 0, 28, 33, 0, datetimeutil.utc),
            datetimeutil.parse_iso_datetime_string('2014-01-13 00:28:33Z')
        )
        self.assertEqual(
            datetime.datetime(2014, 1, 13, 0, 28, 33, 0, datetimeutil.utc),
            datetimeutil.parse_iso_datetime_string('2014-01-13 00:28:33 GMT')
        )

    def test_invalid_values(self):
        self.assertRaises(ValueError, datetimeutil.parse_iso_datetime_string, 123)
        self.assertRaises(ValueError, datetimeutil.parse_iso_datetime_string, '2014-1-13T00:28:33Z')
        self.assertRaises(ValueError, datetimeutil.parse_iso_datetime_string, '2014/1/13T00:28:33Z')
        self.assertRaises(ValueError, datetimeutil.parse_iso_datetime_string, '2014-01-13T00:28:33EST')


class ToDateStringTestCase(unittest.TestCase):
    def test_naive_datetime(self):
        dt = datetime.datetime(2013, 7, 13, 16, 54, 46, 123000)
        actual = datetimeutil.to_ecma_datetime_string(dt, datetimeutil.utc)  # assume UTC to simplify this particular test
        self.assertEqual("2013-07-13T16:54:46.123Z", actual)

    def test_aware_datetime(self):
        dt = datetime.datetime(2013, 7, 13, 16, 54, 46, 123000, datetimeutil.utc)
        actual = datetimeutil.to_ecma_datetime_string(dt, datetimeutil.utc)
        self.assertEqual("2013-07-13T16:54:46.123Z", actual)


class ParseHttpDateStringTestCase(unittest.TestCase):
    def test_valid_values(self):
        self.assertEqual(
            datetime.datetime(2012, 8, 29, 17, 12, 58, tzinfo=datetimeutil.FixedTimezone.from_hours_minutes(10, 30)),
            datetimeutil.parse_http_datetime_string('Wed Aug 29 17:12:58 +1030 2012')
        )
        self.assertEqual(
            datetime.datetime(2012, 8, 29, 17, 12, 58, tzinfo=datetimeutil.FixedTimezone.from_hours_minutes(8, 00)),
            datetimeutil.parse_http_datetime_string('Wed Aug 29 17:12:58 +0800 2012')
        )
        self.assertEqual(
            datetime.datetime(2012, 8, 29, 17, 12, 58, tzinfo=datetimeutil.FixedTimezone.from_hours_minutes(-9, 0)),
            datetimeutil.parse_http_datetime_string('Wed Aug 29 17:12:58 -0900 2012')
        )
        self.assertEqual(
            datetime.datetime(2012, 8, 29, 17, 12, 58, tzinfo=datetimeutil.FixedTimezone.from_hours_minutes(-9, -30)),
            datetimeutil.parse_http_datetime_string('Wed Aug 29 17:12:58 -0930 2012')
        )
        # Alternate position for the year
        self.assertEqual(
            datetime.datetime(2012, 8, 29, 17, 12, 58, tzinfo=datetimeutil.FixedTimezone.from_hours_minutes(-9, -30)),
            datetimeutil.parse_http_datetime_string('Wed, Aug 29 2012 17:12:58 -0930')
        )

    def test_invalid_values(self):
        self.assertRaises(ValueError, datetimeutil.parse_http_datetime_string, 123)
        self.assertRaises(ValueError, datetimeutil.parse_http_datetime_string, '2014-1-13T00:28:33Z')
        self.assertRaises(ValueError, datetimeutil.parse_http_datetime_string, '2014/1/13T00:28:33Z')
        self.assertRaises(ValueError, datetimeutil.parse_http_datetime_string, '2014-01-13T00:28:33EST')





import warnings
warnings.warn(message="This is a ALPHA that is a work in progress (only works with Python 3).")


# Check required libraries are installed
def import_check():
    try:
        import humanfriendly  # noqa
    except ImportError:
        raise ImportError("The `humanfriendly` library is required to use the inspector.")


import_check()



# -*- coding: utf-8 -*-
import unittest
import datetime
from odin import datetimeutil
from odin import serializers


class SerializerTestCase(unittest.TestCase):
    def test_date_iso_format(self):
        self.assertEqual("2014-01-16", serializers.date_iso_format(datetime.date(2014, 1, 16)))

    def test_time_iso_format(self):
        self.assertEqual(
            "00:29:32+00:00",
            serializers.TimeIsoFormat(datetimeutil.utc)(
                datetime.time(0, 29, 32)))
        self.assertEqual(
            "00:29:32+10:00",
            serializers.TimeIsoFormat(datetimeutil.utc)(
                datetime.time(0, 29, 32, tzinfo=datetimeutil.FixedTimezone.from_hours_minutes(10, 0))))

        self.assertEqual(
            "00:29:32-03:30",
            serializers.TimeIsoFormat(datetimeutil.FixedTimezone.from_hours_minutes(-3, -30))(
                datetime.time(0, 29, 32)))
        self.assertEqual(
            "00:29:32+10:00",
            serializers.TimeIsoFormat(datetimeutil.FixedTimezone.from_hours_minutes(-3, -30))(
                datetime.time(0, 29, 32, tzinfo=datetimeutil.FixedTimezone.from_hours_minutes(10, 0))))

    def test_datetime_iso_format(self):
        self.assertEqual(
            "2014-01-16T00:29:32+00:00",
            serializers.DatetimeIsoFormat(datetimeutil.utc)(
                datetime.datetime(2014, 1, 16, 0, 29, 32)))
        self.assertEqual(
            "2014-01-16T00:29:32+10:00",
            serializers.DatetimeIsoFormat(datetimeutil.utc)(
                datetime.datetime(2014, 1, 16, 0, 29, 32, tzinfo=datetimeutil.FixedTimezone.from_hours_minutes(10, 0))))

        self.assertEqual(
            "2014-01-16T00:29:32-03:30",
            serializers.DatetimeIsoFormat(datetimeutil.FixedTimezone.from_hours_minutes(-3, -30))(
                datetime.datetime(2014, 1, 16, 0, 29, 32)))
        self.assertEqual(
            "2014-01-16T00:29:32+10:00",
            serializers.DatetimeIsoFormat(datetimeutil.FixedTimezone.from_hours_minutes(-3, -30))(
                datetime.datetime(2014, 1, 16, 0, 29, 32, tzinfo=datetimeutil.FixedTimezone.from_hours_minutes(10, 0))))

    def test_datetime_ecma_format(self):
        self.assertEqual(
            "2014-01-16T00:29:32.000Z",
            serializers.DatetimeEcmaFormat(False, default_timezone=datetimeutil.utc)(
                datetime.datetime(2014, 1, 16, 0, 29, 32)))
        self.assertEqual(
            "2014-01-15T14:29:32.000Z",
            serializers.DatetimeEcmaFormat(False, default_timezone=datetimeutil.utc)(
                datetime.datetime(2014, 1, 16, 0, 29, 32, tzinfo=datetimeutil.FixedTimezone.from_hours_minutes(10, 0))))

        self.assertEqual(
            "2014-01-16T03:59:32.000Z",
            serializers.DatetimeEcmaFormat(False,
                                           default_timezone=datetimeutil.FixedTimezone.from_hours_minutes(-3, -30))(
                datetime.datetime(2014, 1, 16, 0, 29, 32)))
        self.assertEqual(
            "2014-01-15T14:29:32.000Z",
            serializers.DatetimeEcmaFormat(False,
                                           default_timezone=datetimeutil.FixedTimezone.from_hours_minutes(-3, -30))(
                datetime.datetime(2014, 1, 16, 0, 29, 32, tzinfo=datetimeutil.FixedTimezone.from_hours_minutes(10, 0))))




# -*- coding: utf-8 -*-
import math
import six

__all__ = ('latitude', 'longitude', 'latlng', 'point')


def to_dms(value, absolute=False):
    """
    Split a float value into DMS (degree, minute, second) parts

    :param value: Float value to split
    :param absolute: Obtain the absolute value
    :return: tuple containing DMS values

    """
    invert = not absolute and value < 0
    value = abs(value)
    degree = int(math.floor(value))
    value = (value - degree) * 60
    minute = int(math.floor(value))
    second = (value - minute) * 60
    return (-degree if invert else degree), minute, second


def to_dm(value, absolute=False):
    """
    Split a float value into DM (degree, minute) parts

    :param value: Float value to split
    :param absolute: Obtain the absolute value
    :return: tuple containing DM values

    """
    invert = not absolute and value < 0
    value = abs(value)
    degree = int(math.floor(value))
    minute = (value - degree) * 60
    return (-degree if invert else degree), minute


class latitude(float):  # NoQA
    """
    A latitude value. A latitude is a value between -90.0 and 90.0.
    """
    def __new__(cls, x):
        lat = float.__new__(cls, x)
        if lat > 90.0 or lat < -90.0:
            raise ValueError("not in range -90.0 -> 90.0: '%s'" % x)
        return lat

    def __unicode__(self):
        result = u"%02i°%02i'%02f\"" % to_dms(self, True)
        return result + (u'S' if self < 0 else u'N')

    __str__ = __unicode__ if six.PY3 else float.__str__


class longitude(float):  # NoQA
    """
    A longitude value. A longitude is a value between -180.0 and 180.0.
    """
    def __new__(cls, x):
        lng = float.__new__(cls, x)
        if lng > 180.0 or lng < -180.0:
            raise ValueError("not in range -180.0 -> 180.0: '%s'" % x)
        return lng

    def __unicode__(self):
        result = u"%03i°%02i\'%02f\"" % to_dms(self, True)
        return result + (u'W' if self < 0 else u'E')

    __str__ = __unicode__ if six.PY3 else float.__str__


class latlng(tuple):
    """
    Combination latitude and longitude value.
    """
    def __new__(cls, *args):
        # Unpack a tuple, list or latlng instance.
        if len(args) == 1 and isinstance(args[0], (tuple, list)):
            args = args[0]
        try:
            lat, lng = args
        except ValueError:
            raise TypeError('latlng() takes 2 arguments (%s given)' % len(args))
        return tuple.__new__(cls, (latitude(lat), longitude(lng)))

    @property
    def lat(self):
        return self[0]

    @property
    def lng(self):
        return self[1]

    def __repr__(self):
        return "latlng<%02.4f, %03.4f>" % self

    def __unicode__(self):
        return u"(%s, %s)" % self

    __str__ = __unicode__ if six.PY3 else tuple.__str__


class point(tuple):
    """
    A point in cartesian space. This type can be either 2D (on a plain) or 3D (includes a z-axis).
    """
    def __new__(cls, *args):
        if len(args) == 1 and isinstance(args[0], (tuple, list)):
            args = args[0]
        if len(args) in (2, 3):
            return tuple.__new__(cls, (float(x) for x in args))
        else:
            raise TypeError('point() takes 2-3 arguments (%s given)' % len(args))

    @property
    def x(self):
        return self[0]

    @property
    def y(self):
        return self[1]

    @property
    def z(self):
        try:
            return self[2]
        except IndexError:
            raise AttributeError('2D points do not include a z axis.')

    @property
    def is_3d(self):
        return len(self) == 3

    def __repr__(self):
        return ("point<%f, %f, %f>" if self.is_3d else "point<%f, %f>") % self

    def __str__(self):
        return ("(%f, %f, %f)" if self.is_3d else "(%f, %f)") % self



from __future__ import absolute_import
import unittest
import datetime
from odin.codecs import dict_codec
from odin.datetimeutil import utc
from .resources import *


class DictCodecTestCase(unittest.TestCase):
    def test_dump(self):
        in_resource = Book(
            title='Consider Phlebas',
            isbn='0-333-45430-8',
            num_pages=471,
            rrp=19.50,
            fiction=True,
            genre="sci-fi",
            authors=[Author(name="Iain M. Banks")],
            publisher=Publisher(name="Macmillan"),
        )

        actual = dict_codec.dump(in_resource)
        self.assertDictEqual(actual, {
            '$': 'library.Book',
            'authors': [{
                '$': 'Author',
                'name': "Iain M. Banks",
            }],
            'fiction': True,
            'genre': "sci-fi",
            'num_pages': 471,
            'rrp': 19.50,
            'isbn': '0-333-45430-8',
            'title': 'Consider Phlebas',
            'published': [],
            'publisher': {
                '$': 'Publisher',
                'name': "Macmillan",
            },
        })

    def test_dump_no_type_field(self):
        in_resource = Book(
            title='Consider Phlebas',
            isbn='0-333-45430-8',
            num_pages=471,
            rrp=19.50,
            fiction=True,
            genre="sci-fi",
            authors=[Author(name="Iain M. Banks")],
            publisher=Publisher(name="Macmillan"),
        )

        actual = dict_codec.dump(in_resource, include_type_field=False)
        self.assertDictEqual(actual, {
            'authors': [{
                'name': "Iain M. Banks",
            }],
            'fiction': True,
            'genre': "sci-fi",
            'num_pages': 471,
            'rrp': 19.50,
            'isbn': '0-333-45430-8',
            'title': 'Consider Phlebas',
            'published': [],
            'publisher': {
                'name': "Macmillan",
            },
        })

    def test_dump_and_load_(self):
        in_resource = Book(
            title='Consider Phlebas',
            isbn='0-333-45430-8',
            num_pages=471,
            rrp=19.50,
            fiction=True,
            genre="sci-fi",
            authors=[Author(name="Iain M. Banks")],
            publisher=Publisher(name="Macmillan"),
            published=[datetime.datetime(1987, 1, 1, tzinfo=utc)]
        )

        d = dict_codec.dump(in_resource)
        out_resource = dict_codec.load(d)

        self.assertEqual(out_resource.title, in_resource.title)
        self.assertEqual(out_resource.isbn, in_resource.isbn)
        self.assertEqual(out_resource.num_pages, in_resource.num_pages)
        self.assertEqual(out_resource.rrp, in_resource.rrp)
        self.assertEqual(out_resource.fiction, in_resource.fiction)
        self.assertEqual(out_resource.genre, in_resource.genre)
        self.assertEqual(out_resource.authors[0].name, in_resource.authors[0].name)
        self.assertEqual(out_resource.publisher.name, in_resource.publisher.name)
        self.assertEqual(out_resource.published[0], in_resource.published[0])



# -*- coding: utf-8 -*-
from __future__ import absolute_import
import datetime
import os
from odin.datetimeutil import utc
from six import BytesIO
import unittest
from odin.codecs import msgpack_codec
from .resources import *

FIXTURE_PATH_ROOT = os.path.join(os.path.dirname(__file__), "fixtures")


class MsgPackCodecTestCase(unittest.TestCase):
    def test_dumps_and_loads(self):
        in_resource = Book(
            title='Consider Phlebas',
            isbn='0-333-45430-8',
            num_pages=471,
            rrp=19.50,
            fiction=True,
            genre="sci-fi",
            authors=[Author(name="Iain M. Banks")],
            publisher=Publisher(name="Macmillan"),
        )

        data = msgpack_codec.dumps(in_resource)
        out_resource = msgpack_codec.loads(data)

        self.assertEqual(out_resource.title, in_resource.title)
        self.assertEqual(out_resource.isbn, in_resource.isbn)
        self.assertEqual(out_resource.num_pages, in_resource.num_pages)
        self.assertEqual(out_resource.rrp, in_resource.rrp)
        self.assertEqual(out_resource.fiction, in_resource.fiction)
        self.assertEqual(out_resource.genre, in_resource.genre)
        self.assertEqual(out_resource.authors[0].name, in_resource.authors[0].name)
        self.assertEqual(out_resource.publisher.name, in_resource.publisher.name)

    def test_dump_and_load_(self):
        in_resource = Book(
            title='Consider Phlebas',
            isbn='0-333-45430-8',
            num_pages=471,
            rrp=19.50,
            fiction=True,
            genre="sci-fi",
            authors=[Author(name="Iain M. Banks")],
            publisher=Publisher(name="Macmillan"),
            published=[datetime.datetime(1987, 1, 1, tzinfo=utc)]
        )

        fp = BytesIO()
        msgpack_codec.dump(in_resource, fp)

        fp.seek(0)
        out_resource = msgpack_codec.load(fp)

        self.assertEqual(out_resource.title, in_resource.title)
        self.assertEqual(out_resource.isbn, in_resource.isbn)
        self.assertEqual(out_resource.num_pages, in_resource.num_pages)
        self.assertEqual(out_resource.rrp, in_resource.rrp)
        self.assertEqual(out_resource.fiction, in_resource.fiction)
        self.assertEqual(out_resource.genre, in_resource.genre)
        self.assertEqual(out_resource.authors[0].name, in_resource.authors[0].name)
        self.assertEqual(out_resource.publisher.name, in_resource.publisher.name)
        self.assertEqual(out_resource.published[0], in_resource.published[0])



# -*- coding: utf-8 -*-
"""
Do load/dump tests on known valid and invalid documents.
"""
import os
import datetime

from odin.codecs import json_codec
from odin import exceptions
from odin.datetimeutil import utc

from ._test_case import TestCase
from .resources import *


FIXTURE_PATH_ROOT = os.path.join(os.path.dirname(__file__), "fixtures")


class KitchenSinkTestCase(TestCase):
    def test_dumps_with_valid_data(self):
        book = Book(title="Consider Phlebas", isbn="0-333-45430-8", num_pages=471, rrp=19.50, genre="sci-fi", fiction=True,
                    published=datetime.datetime(1987, 1, 1, tzinfo=utc))
        book.publisher = Publisher(name="Macmillan")
        book.authors.append(Author(name="Iain M. Banks"))

        library = Library(name="Public Library", books=[book])
        actual = json_codec.dumps(library)

        self.assertJSONEqual("""
{
    "$": "Library",
    "name": "Public Library",
    "books": [
        {
            "$": "library.Book",
            "publisher": {
                "$": "Publisher",
                "name": "Macmillan"
            },
            "num_pages": 471,
            "isbn": "0-333-45430-8",
            "title": "Consider Phlebas",
            "authors": [
                {
                    "$": "Author",
                    "name": "Iain M. Banks"
                }
            ],
            "fiction": true,
            "published": "1987-01-01T00:00:00+00:00",
            "genre": "sci-fi",
            "rrp": 19.5
        }
    ],
    "subscribers": [],
    "book_count": 1
}
        """, actual)

    def test_full_clean_invalid_data(self):
        book = Book(title="Consider Phlebas", num_pages=471, rrp=19.50, genre="space opera", fiction=True)
        book.publisher = Publisher(name="Macmillan")
        book.authors.append(Author(name="Iain M. Banks"))

        library = Library(name="Public Library", books=[book])

        with self.assertRaises(exceptions.ValidationError):
            library.full_clean()

    def test_load_valid_data(self):
        library = json_codec.load(open(os.path.join(FIXTURE_PATH_ROOT, "book-valid.json")))

        self.assertEqual("Consider Phlebas", library.books[0].title)

    def test_load_invalid_data(self):
        with self.assertRaises(exceptions.ValidationError):
            json_codec.load(open(os.path.join(FIXTURE_PATH_ROOT, "book-invalid.json")))



# -*- coding: utf-8 -*-
import copy
import six
from odin import exceptions, registration
from odin.exceptions import ValidationError
from odin.fields import NOT_PROVIDED
from odin.utils import cached_property, field_iter_items


DEFAULT_TYPE_FIELD = '$'
META_OPTION_NAMES = (
    'name', 'namespace', 'name_space', 'verbose_name', 'verbose_name_plural', 'abstract', 'doc_group', 'type_field',
    'key_field_name'
)


class ResourceOptions(object):
    def __init__(self, meta):
        self.meta = meta
        self.parents = []
        self.fields = []
        self.virtual_fields = []

        self.name = None
        self.class_name = None
        self.name_space = NOT_PROVIDED
        self.verbose_name = None
        self.verbose_name_plural = None
        self.abstract = False
        self.doc_group = None
        self.type_field = DEFAULT_TYPE_FIELD
        self.key_field_name = None

        self._cache = {}

    def contribute_to_class(self, cls, name):
        cls._meta = self
        self.name = cls.__name__
        self.class_name = "%s.%s" % (cls.__module__, cls.__name__)

        if self.meta:
            meta_attrs = self.meta.__dict__.copy()
            for name in self.meta.__dict__:
                if name.startswith('_'):
                    del meta_attrs[name]
            for attr_name in META_OPTION_NAMES:
                if attr_name in meta_attrs:
                    # Allow meta to be defined as namespace
                    if attr_name == 'namespace':
                        setattr(self, 'name_space', meta_attrs.pop(attr_name))
                    else:
                        setattr(self, attr_name, meta_attrs.pop(attr_name))
                elif hasattr(self.meta, attr_name):
                    setattr(self, attr_name, getattr(self.meta, attr_name))

            # Any leftover attributes must be invalid.
            if meta_attrs != {}:
                raise TypeError("'class Meta' got invalid attribute(s): %s" % ','.join(meta_attrs.keys()))
        del self.meta

        if not self.verbose_name:
            self.verbose_name = self.name.replace('_', ' ').strip('_ ')
        if not self.verbose_name_plural:
            self.verbose_name_plural = self.verbose_name + 's'

    def add_field(self, field):
        self.fields.append(field)
        cached_property.clear_caches(self)

    def add_virtual_field(self, field):
        self.virtual_fields.append(field)
        cached_property.clear_caches(self)

    def get_key_field(self):
        return self.field_map.get(self.key_field)

    @property
    def resource_name(self):
        """
        Full name of resource including namespace (if specified)
        """
        if self.name_space:
            return "%s.%s" % (self.name_space, self.name)
        else:
            return self.name

    @cached_property
    def all_fields(self):
        """
        All fields both standard and virtual.
        """
        return self.fields + self.virtual_fields

    @cached_property
    def composite_fields(self):
        """
        All composite fields.
        """
        # Not the nicest solution but is a fairly safe way of detecting a composite field.
        return [f for f in self.fields if (hasattr(f, 'of') and issubclass(f.of, Resource))]

    @cached_property
    def container_fields(self):
        """
        All composite fields with the container flag.

        Used by XML like codecs.

        """
        return [f for f in self.composite_fields if getattr(f, 'use_container', False)]

    @cached_property
    def field_map(self):
        return {f.attname: f for f in self.fields}

    @cached_property
    def parent_resource_names(self):
        """
        List of parent resource names.
        """
        return [p._meta.resource_name for p in self.parents]

    @cached_property
    def attribute_fields(self):
        """
        List of fields where is_attribute is True.
        """
        return [f for f in self.fields if f.is_attribute]

    @cached_property
    def element_fields(self):
        """
        List of fields where is_attribute is False.
        """
        return [f for f in self.fields if not f.is_attribute]

    @cached_property
    def element_field_map(self):
        return {f.attname: f for f in self.element_fields}

    @cached_property
    def key_field(self):
        """
        Field specified as the key field
        """
        return self.field_map.get(self.key_field_name)

    def __repr__(self):
        return '<Options for %s>' % self.resource_name


class ResourceBase(type):
    """
    Metaclass for all Resources.
    """
    def __new__(cls, name, bases, attrs):
        super_new = super(ResourceBase, cls).__new__

        # attrs will never be empty for classes declared in the standard way
        # (ie. with the `class` keyword). This is quite robust.
        if name == 'NewBase' and attrs == {}:
            return super_new(cls, name, bases, attrs)

        parents = [
            b for b in bases if
            isinstance(b, ResourceBase) and not (b.__name__ == 'NewBase' and b.__mro__ == (b, object))
        ]
        if not parents:
            # If this isn't a subclass of Resource, don't do anything special.
            return super_new(cls, name, bases, attrs)

        # Create the class.
        module = attrs.pop('__module__')
        new_class = super_new(cls, name, bases, {'__module__': module})
        attr_meta = attrs.pop('Meta', None)
        abstract = getattr(attr_meta, 'abstract', False)
        if not attr_meta:
            meta = getattr(new_class, 'Meta', None)
        else:
            meta = attr_meta
        base_meta = getattr(new_class, '_meta', None)

        new_class.add_to_class('_meta', ResourceOptions(meta))

        # Generate a namespace if one is not provided
        if new_class._meta.name_space is NOT_PROVIDED and base_meta:
            # Namespace is inherited
            if (not new_class._meta.name_space) or (new_class._meta.name_space is NOT_PROVIDED):
                new_class._meta.name_space = base_meta.name_space

        if new_class._meta.name_space is NOT_PROVIDED:
            new_class._meta.name_space = module

        # Key field is inherited
        if base_meta and (not new_class._meta.key_field_name) or (new_class._meta.key_field_name is NOT_PROVIDED):
            new_class._meta.key_field_name = base_meta.key_field_name

        # Bail out early if we have already created this class.
        r = registration.get_resource(new_class._meta.resource_name)
        if r is not None:
            return r

        # Add all attributes to the class.
        for obj_name, obj in attrs.items():
            new_class.add_to_class(obj_name, obj)

        # Sort the fields
        new_class._meta.fields = sorted(new_class._meta.fields, key=hash)

        # All the fields of any type declared on this model
        local_field_attnames = set([f.attname for f in new_class._meta.fields])
        field_attnames = set(local_field_attnames)

        for base in parents:
            if not hasattr(base, '_meta'):
                # Things without _meta aren't functional models, so they're
                # uninteresting parents.
                continue

            # Check for clashes between locally declared fields and those
            # on the base classes (we cannot handle shadowed fields at the
            # moment).
            for field in base._meta.all_fields:
                if field.attname in local_field_attnames:
                    raise Exception('Local field %r in class %r clashes with field of similar name from '
                                    'base class %r' % (field.attname, name, base.__name__))
            for field in base._meta.fields:
                if field.attname not in field_attnames:
                    field_attnames.add(field.attname)
                    new_class.add_to_class(field.attname, copy.deepcopy(field))
            for field in base._meta.virtual_fields:
                new_class.add_to_class(field.attname, copy.deepcopy(field))

            new_class._meta.parents += base._meta.parents
            new_class._meta.parents.append(base)

        # If a key_field is defined ensure it exists
        if new_class._meta.key_field_name is not None and new_class._meta.key_field is None:
                raise AttributeError('Key field `{}` is not exist on this resource.'.format(
                    new_class._meta.key_field_name)
                )

        if abstract:
            return new_class

        # Register resource
        registration.register_resources(new_class)

        # Because of the way imports happen (recursively), we may or may not be
        # the first time this model tries to register with the framework. There
        # should only be one class for each model, so we always return the
        # registered version.
        return registration.get_resource(new_class._meta.resource_name)

    def add_to_class(cls, name, value):
        if hasattr(value, 'contribute_to_class'):
            value.contribute_to_class(cls, name)
        else:
            setattr(cls, name, value)


@six.add_metaclass(ResourceBase)
class Resource(object):
    def __init__(self, *args, **kwargs):
        args_len = len(args)
        if args_len > len(self._meta.fields):
            raise TypeError('This resource takes %s positional arguments but %s where given.' % (
                len(self._meta.fields), args_len))

        # The ordering of the zip calls matter - zip throws StopIteration
        # when an iter throws it. So if the first iter throws it, the second
        # is *not* consumed. We rely on this, so don't change the order
        # without changing the logic.
        fields_iter = iter(self._meta.fields)
        if args_len:
            if not kwargs:
                for val, field in zip(args, fields_iter):
                    setattr(self, field.attname, val)
            else:
                for val, field in zip(args, fields_iter):
                    setattr(self, field.attname, val)
                    kwargs.pop(field.name, None)

        # Now we're left with the unprocessed fields that *must* come from
        # keywords, or default.
        for field in fields_iter:
            try:
                val = kwargs.pop(field.attname)
            except KeyError:
                val = field.get_default()
            setattr(self, field.attname, val)

        if kwargs:
            raise TypeError("'%s' is an invalid keyword argument for this function" % list(kwargs)[0])

    def __repr__(self):
        return '<%s: %s>' % (self.__class__.__name__, self)

    def __str__(self):
        return '%s resource' % self._meta.resource_name

    @classmethod
    def create_from_dict(cls, d, full_clean=False):
        """
        Create a resource instance from a dictionary.
        """
        return create_resource_from_dict(d, cls, full_clean)

    def to_dict(self, include_virtual=True):
        """
        Convert this resource into a `dict` of field_name/value pairs.

        .. note::
            This method is not recursive, it only operates on this single resource, any sub resources are returned as
            is. The use case that prompted the creation of this method is within codecs when a resource must be
            converted into a type that can be serialised, these codecs then operate recursively on the returned `dict`.

        :param include_virtual: Include virtual fields when generating `dict`.

        """
        fields = self._meta.all_fields if include_virtual else self._meta.fields
        return dict((f.name, v) for f, v in field_iter_items(self, fields))

    def convert_to(self, to_resource, context=None, ignore_fields=None, **field_values):
        """
        Convert this resource into a specified resource.

        A mapping must be defined for conversion between this resource and to_resource or an exception will be raised.

        """
        mapping = registration.get_mapping(self.__class__, to_resource)
        ignore_fields = ignore_fields or []
        ignore_fields.extend(mapping.exclude_fields)
        self.full_clean(ignore_fields)
        return mapping(self, context).convert(**field_values)

    def update_existing(self, dest_obj, context=None, ignore_fields=None):
        """
        Update the fields on an existing destination object.

        A mapping must be defined for conversion between this resource and ``dest_obj`` type or an exception will be
        raised.

        """
        self.full_clean(ignore_fields)
        mapping = registration.get_mapping(self.__class__, dest_obj.__class__)
        return mapping(self, context).update(dest_obj, ignore_fields)

    def extra_attrs(self, attrs):
        """
        Called during de-serialisation of data if there are any extra fields defined in the document.

        This allows the resource to decide how to handle these fields. By default they are ignored.
        """
        pass

    def clean(self):
        """
        Chance to do more in depth validation.
        """
        pass

    def full_clean(self, exclude=None):
        """
        Calls clean_fields, clean on the resource and raises ``ValidationError``
        for any errors that occurred.
        """
        errors = {}

        try:
            self.clean_fields(exclude)
        except ValidationError as e:
            errors = e.update_error_dict(errors)

        try:
            self.clean()
        except ValidationError as e:
            errors = e.update_error_dict(errors)

        if errors:
            raise ValidationError(errors)

    def clean_fields(self, exclude=None):
        errors = {}

        for f in self._meta.fields:
            if exclude and f.name in exclude:
                continue

            raw_value = f.value_from_object(self)

            if f.null and raw_value is None:
                continue

            try:
                raw_value = f.clean(raw_value)
            except ValidationError as e:
                errors[f.name] = e.messages

            # Check for resource level clean methods.
            clean_method = getattr(self, "clean_%s" % f.attname, None)
            if callable(clean_method):
                try:
                    raw_value = clean_method(raw_value)
                except ValidationError as e:
                    errors.setdefault(f.name, []).extend(e.messages)

            setattr(self, f.attname, raw_value)

        if errors:
            raise ValidationError(errors)


def resolve_resource_type(resource):
    if isinstance(resource, type) and issubclass(resource, Resource):
        return resource._meta.resource_name, resource._meta.type_field
    else:
        return resource, DEFAULT_TYPE_FIELD


def create_resource_from_dict(d, resource=None, full_clean=True, copy_dict=True):
    """
    Create a resource from a dict.

    :param d: dictionary of data.
    :param resource: A resource type, resource name or list of resources and names to use as the base for creating a
        resource. If a list is supplied the first item will be used if a resource type is not supplied; this could also
        be a parent(s) of any resource defined by the dict.
    :param full_clean: Do a full clean as part of the creation.
    :param copy_dict: Use a copy of the input dictionary rather than destructively processing the input dict.

    """
    assert isinstance(d, dict)

    if copy_dict:
        d = d.copy()

    if resource:
        resource_type = None

        # Convert to single resource then resolve document type
        if isinstance(resource, (tuple, list)):
            resources = (resolve_resource_type(r) for r in resource)
        else:
            resources = [resolve_resource_type(resource)]

        for resource_name, type_field in resources:
            # See if the input includes a type field  and check it's registered
            document_resource_name = d.get(type_field, None)
            if document_resource_name:
                resource_type = registration.get_resource(document_resource_name)
            else:
                resource_type = registration.get_resource(resource_name)

            if not resource_type:
                raise exceptions.ResourceException("Resource `%s` is not registered." % document_resource_name)

            if document_resource_name:
                # Check resource types match or are inherited types
                if (resource_name == document_resource_name or
                        resource_name in resource_type._meta.parent_resource_names):
                    break  # We are done
            else:
                break

        if not resource_type:
            raise exceptions.ResourceException(
                "Incoming resource does not match [%s]" % ', '.join(r for r, t in resources))
    else:
        # No resource specified, relay on type field
        document_resource_name = d.pop(DEFAULT_TYPE_FIELD, None)
        if not document_resource_name:
            raise exceptions.ResourceException("Resource not defined.")

        # Get an instance of a resource type
        resource_type = registration.get_resource(document_resource_name)
        if not resource_type:
            raise exceptions.ResourceException("Resource `%s` is not registered." % document_resource_name)

    attrs = []
    errors = {}
    for f in resource_type._meta.fields:
        value = d.pop(f.name, NOT_PROVIDED)
        if value is NOT_PROVIDED:
            value = f.get_default() if f.use_default_if_not_provided else None
        else:
            try:
                value = f.to_python(value)
            except ValidationError as ve:
                errors[f.name] = ve.error_messages
        attrs.append(value)

    if errors:
        raise ValidationError(errors)

    new_resource = resource_type(*attrs)
    if d:
        new_resource.extra_attrs(d)
    if full_clean:
        new_resource.full_clean()
    return new_resource


def build_object_graph(d, resource=None, full_clean=True, copy_dict=True):
    """
    Generate an object graph from a dict

    :param d: Dictionary to build from
    :param resource: A resource type, resource name or list of resources and names to use as the base for creating a
        resource. If a list is supplied the first item will be used if a resource type is not supplied.
    :param full_clean: Perform a full clean once built; default is True
    :param copy_dict: Clone the dict before doing build; default is True
    :raises ValidationError: During building of the object graph and issues discovered are raised as a ValidationError.

    """
    if isinstance(d, dict):
        return create_resource_from_dict(d, resource, full_clean, copy_dict)

    if isinstance(d, list):
        return [build_object_graph(o, resource, full_clean, copy_dict) for o in d]

    return d


class ResourceIterable(object):
    """
    Iterable that yields resources.
    """
    def __init__(self, sequence):
        self.sequence = sequence

    def __iter__(self):
        for item in self.sequence:
            yield item



# -*- coding: utf-8 -*-
from odin.utils import cached_property, field_iter_items

__all__ = ('ResourceAdapter',)


class CurriedAdapter(object):
    """
    Curry wrapper for a Adapter to allow for pre-config of include/exclude and
    any other user defined arguments provided in kwargs.
    """
    def __init__(self, cls, **kwargs):
        self.cls = cls
        self.kwargs = kwargs.copy()

    def __call__(self, source):
        return self.cls(source, **self.kwargs)

    def apply_to(self, sources):
        return self.cls.apply_to(sources, **self.kwargs)


class ResourceOptionsAdapter(object):
    """
    A lightweight wrapper for the *ResourceOptions* class that filters fields.
    """
    def __init__(self, options, include, exclude):
        self._wrapped = options
        self.parents = options.parents

        # Filter available fields
        include = include or [f.attname for f in options.all_fields]
        exclude = exclude or []
        self.fields = [f for f in options.fields if f.attname in include and f.attname not in exclude]
        self.virtual_fields = [f for f in options.virtual_fields if f.attname in include and f.attname not in exclude]

        # Work around so cached properties still work.
        self._cache = {}

    def __getattr__(self, item):
        return getattr(self._wrapped, item)

    def __repr__(self):
        return '<Options Adapter for %s>' % self.resource_name

    @cached_property
    def all_fields(self):
        """
        All fields both standard and virtual.
        """
        return self.fields + self.virtual_fields

    @cached_property
    def field_map(self):
        return {f.attname: f for f in self.fields}

    @property
    def attribute_fields(self):
        """
        List of fields where is_attribute is True.
        """
        return [f for f in self.fields if f.is_attribute]

    @property
    def element_fields(self):
        """
        List of fields where is_attribute is False.
        """
        return [f for f in self.fields if not f.is_attribute]


class ResourceAdapter(object):
    """
    A lightweight wrapper that can be placed around a resource to filter out specific
    fields or to provide additional specific methods or calculated properties.

    A good use case for an adapter is an API where you wish to filter out certain fields
    or in a UI where you wish to add methods for rendering objects without extending the
    underlying resource that may be shared between multiple rendering engines or other
    subsystems.

    The *ResourceAdapter* can be passed to Odin codecs just like a *Resource*.

    """
    @classmethod
    def apply_to(cls, sources, include=None, exclude=None, **kwargs):
        """
        Convenience method that applies include/exclude lists to all items in an
        iterable collection of resources.

        :param sources: Source resources being wrapped.
        :param include: Fields that should be explicitly included on the adapter.
        :param exclude: Fields to explicitly exclude on the adapter.

        """
        meta_objects = {}
        for resource in sources:
            try:
                meta = meta_objects[resource._meta.resource_name]
            except KeyError:
                meta = cls._create_options_adapter(resource._meta, include, exclude)
                meta_objects[resource._meta.resource_name] = meta
            yield cls(resource, meta=meta, **kwargs)

    @classmethod
    def _create_options_adapter(cls, options, include=None, exclude=None):
        include_fields = include if include else getattr(cls, 'include_fields', None)
        exclude_fields = exclude if exclude else getattr(cls, 'exclude_fields', None)
        return ResourceOptionsAdapter(options, include_fields, exclude_fields)

    @classmethod
    def curry(cls, include=None, exclude=None, **kwargs):
        """
        Creates an Adapter that has include and exclude (and any other options) preset.

        :param include: Fields that should be explicitly included on the adapter.
        :param exclude: Fields to explicitly exclude on the adapter.
        :param kwargs:

        """
        return CurriedAdapter(cls, include=include, exclude=exclude, **kwargs)

    def __init__(self, source, include=None, exclude=None, meta=None):
        """
        Initialise the adapter.

        :param source: Source resource being wrapped.
        :param include: Fields that should be explicitly included on the adapter.
        :param exclude: Fields to explicitly exclude on the adapter.

        """
        self.__dict__['_source'] = source

        if not meta:
            meta = self._create_options_adapter(source._meta, include, exclude)
        self._meta = meta

    def __getattr__(self, item):
        return getattr(self._source, item)

    def __setattr__(self, name, value):
        setattr(self._source, name, value)

    def __repr__(self):
        return '<%s: %s>' % (self.__class__.__name__, self)

    def __str__(self):
        return '%s resource adapter' % self._meta.resource_name

    def to_dict(self, include_virtual=True):
        """
        Convert this resource into a dict
        """
        fields = self._meta.all_fields if include_virtual else self._meta.fields
        return dict((f.name, v) for f, v in field_iter_items(self, fields))



# -*- coding: utf-8 -*-
from odin import registration

NON_FIELD_ERRORS = '__all__'


class ValidationError(Exception):
    """An error while validating data."""
    def __init__(self, message, code=None, params=None):
        """
        ValidationError can be passed any object that can be printed (usually
        a string), a list of objects or a dictionary.
        """
        if isinstance(message, dict):
            self.message_dict = message

        if isinstance(message, list):
            self.messages = message
        else:
            self.messages = [message]
            self.code = code
            self.params = params

    def __str__(self):
        # This is needed because, without a __str__(), printing an exception
        # instance would result in this:
        # AttributeError: ValidationError instance has no attribute 'args'
        # See http://www.python.org/doc/current/tut/node10.html#handling
        if hasattr(self, 'message_dict'):
            message_dict = self.message_dict
            return "{%s}" % ', '.join("'%s': %r" % (key, message_dict[key]) for key in sorted(message_dict))
        return repr(self.messages)

    def __repr__(self):
        return 'ValidationError(%s)' % self

    @property
    def error_messages(self):
        if hasattr(self, 'message_dict'):
            return self.message_dict
        else:
            return self.messages

    def update_error_dict(self, error_dict):
        if hasattr(self, 'message_dict'):
            if error_dict:
                for k, v in self.message_dict.items():
                    error_dict.setdefault(k, []).extend(v)
            else:
                error_dict = self.message_dict
        else:
            error_dict[NON_FIELD_ERRORS] = self.messages
        return error_dict


def validation_error_handler(exception, field, errors):
    if hasattr(exception, 'code') and exception.code in field.error_messages:
        message = field.error_messages[exception.code]
        if exception.params:
            message = message % exception.params
        errors.append(message)
    else:
        errors.extend(exception.messages)
registration.register_validation_error_handler(ValidationError, validation_error_handler)


class ResourceException(ValidationError):
    """
    Errors raised when generating resource from files.

    Exception inherits from ValidationError for backwards compatibility.

    """


class MappingError(Exception):
    """
    Exceptions related to mapping, will typically be a more specific `MappingSetupError` or `MappingExecutionError`.
    """


class MappingSetupError(MappingError):
    """
    Exception raised during the setup of mapping rules.
    """


class MappingExecutionError(MappingError):
    """
    Exception raised during the execution of mapping rules.
    """


class CodecError(Exception):
    """
    Exception raised by a codec during an operation.
    """


class CodecDecodeError(CodecError):
    """
    Exception raised by a codec during a decoding operation.
    """


class CodecEncodeError(CodecError):
    """
    Exception raised by a codec during an encoding operation.
    """


class TraversalError(Exception):
    """
    Exception raised during a traversal operation.
    """
    def __init__(self, path, *args, **kwargs):
        super(TraversalError, self).__init__(*args, **kwargs)
        self.path = path


class InvalidPathError(TraversalError):
    """
    Raised when a path is invalid (eg referencing an unknown field)
    """


class NoMatchError(TraversalError):
    """
    When traversing a path to get a value no match was found.
    """


class MultipleMatchesError(TraversalError):
    """
    When traversing a path to get a single value, a filtering operation matched multiple values.
    """



from __future__ import absolute_import, print_function
from humanfriendly.terminal import ansi_wrap, HIGHLIGHT_COLOR


def resource_reference(resource):
    return "{} <{}.{}>".format(resource._meta.resource_name, resource.__module__, resource.__name__)


class SummaryBase(object):
    """
    Base class for summaries
    """
    def __init__(self, out):
        self.out = out

    def print(self, sep=' ', end='\n', *objects):
        print(*objects, sep=sep, end=end, file=self.out)

    def title(self, value, underline='='):
        self.print(
            ansi_wrap(value, bold=True, color=HIGHLIGHT_COLOR),
            underline * len(value),
            sep='\n'
        )



from __future__ import absolute_import
import unittest
import os
from odin import filtering
from odin.traversal import TraversalPath
from odin.codecs import json_codec


# Open library fixture
with open(os.path.join(os.path.dirname(__file__), 'fixtures', 'library.json')) as f:
    library = json_codec.load(f)

first_book = TraversalPath.parse('books[0]').get_value(library)


class FilterTestCase(unittest.TestCase):
    def test_matches_top_level_resource(self):
        flt = filtering.Equal('title', "Consider Phlebas")
        self.assertTrue(flt(first_book))

    def test_matches_multiple_matches_resource(self):
        # Less than fiction $20 with a single author and not fantasy
        flt = filtering.And(
            filtering.Equal('fiction', True),
            filtering.LessThan('rrp', 20),
            filtering.Equal('authors', 1, len),
            filtering.NotEqual('genre', 'fantasy'),
        )
        self.assertTrue(flt(first_book))

    def test_matches_sub_resource(self):
        flt = filtering.Equal('publisher.name', 'Macmillan')
        self.assertTrue(flt(first_book))

    def test_find_sub_resources(self):
        flt = filtering.Equal('fiction', True)
        # Some fiction?
        self.assertTrue(flt.any(library.books))
        # All fiction
        self.assertFalse(flt.all(library.books))

    def test_resource_field_does_not_exist(self):
        flt = filtering.Equal('title', 'Public Library')
        self.assertFalse(flt(library))


class FilterChainTestCase(unittest.TestCase):
    def test_empty_description(self):
        flt = filtering.And()
        self.assertEqual('', str(flt))

    def test_description(self):
        flt = filtering.And(
            filtering.Equal('fiction', True),
            filtering.LessThan('rrp', 20),
        )
        self.assertEqual(
            '(fiction == True AND rrp < 20)',
            str(flt)
        )

    def test_combining_same_type(self):
        flt1 = filtering.And(
            filtering.Equal('fiction', True),
            filtering.LessThan('rrp', 20),
        )
        flt2 = filtering.And(
            filtering.Equal('authors', 1, len),
            filtering.NotEqual('genre', 'fantasy'),
        )
        flt = flt1 + flt2

        self.assertEqual(
            "(fiction == True AND rrp < 20 AND len(authors) == 1 AND genre != 'fantasy')",
            str(flt)
        )

    def test_combine_with_comparison(self):
        flt1 = filtering.And(
            filtering.Equal('fiction', True),
            filtering.LessThan('rrp', 20),
        )
        flt = flt1 + filtering.Equal('authors', 1, len)

        self.assertEqual(
            "(fiction == True AND rrp < 20 AND len(authors) == 1)",
            str(flt)
        )

    def test_combine_with_other_type(self):
        flt1 = filtering.And(
            filtering.Equal('fiction', True),
            filtering.LessThan('rrp', 20),
        )
        self.assertRaises(TypeError, lambda: flt1 + 'abc')


class FilterComparisonTestCase(unittest.TestCase):
    def test_description(self):
        comp = filtering.FilterComparison('foo', 'bar')
        comp.operator_symbols = ['%']
        self.assertEqual("foo % 'bar'", str(comp))
        comp = filtering.FilterComparison('foo', 42)
        comp.operator_symbols = ['%']
        self.assertEqual('foo % 42', str(comp))
        comp = filtering.FilterComparison('foo', 'bar', any)
        comp.operator_symbols = ['%']
        self.assertEqual("any(foo) % 'bar'", str(comp))

    def test_equal_string(self):
        comp = filtering.Equal('foo', 'bar')
        self.assertTrue(comp.compare('bar'))
        self.assertFalse(comp.compare('eek'))
        self.assertEqual("foo == 'bar'", str(comp))

    def test_equal_integer(self):
        comp = filtering.Equal('foo', 5)
        self.assertTrue(comp.compare(5))
        self.assertFalse(comp.compare(6))
        self.assertEqual('foo == 5', str(comp))

    def test_not_equal(self):
        comp = filtering.NotEqual('foo', 'bar')
        self.assertTrue(comp.compare('eek'))
        self.assertFalse(comp.compare('bar'))
        self.assertEqual("foo != 'bar'", str(comp))

    def test_less_than(self):
        comp = filtering.LessThan('foo', 5)
        self.assertTrue(comp.compare(4))
        self.assertFalse(comp.compare(5))
        self.assertFalse(comp.compare(6))
        self.assertEqual('foo < 5', str(comp))

    def test_less_than_or_equal(self):
        comp = filtering.LessThanOrEqual('foo', 5)
        self.assertTrue(comp.compare(4))
        self.assertTrue(comp.compare(5))
        self.assertFalse(comp.compare(6))
        self.assertEqual('foo <= 5', str(comp))

    def test_greater_than(self):
        comp = filtering.GreaterThan('foo', 5)
        self.assertFalse(comp.compare(4))
        self.assertFalse(comp.compare(5))
        self.assertTrue(comp.compare(6))
        self.assertEqual('foo > 5', str(comp))

    def test_greater_than_or_equal(self):
        comp = filtering.GreaterThanOrEqual('foo', 5)
        self.assertFalse(comp.compare(4))
        self.assertTrue(comp.compare(5))
        self.assertTrue(comp.compare(6))
        self.assertEqual('foo >= 5', str(comp))



# -*- coding: utf-8 -*-
from __future__ import absolute_import
import os
import unittest
from odin.codecs import xml_codec
from .resources import *

FIXTURE_PATH_ROOT = os.path.join(os.path.dirname(__file__), "fixtures")


class XmlLoadTestCase(unittest.TestCase):
    # def test_valid(self):
    #     with open(os.path.join(FIXTURE_PATH_ROOT, "book-valid.xml")) as f:
    #         xml_codec.load(f, Library)

    def test_dumps(self):
        book = Book(
            title='Consider Phlebas & Other stories',
            isbn='0-333-45430-8',
            num_pages=471,
            rrp=19.50,
            fiction=True,
            genre="sci-fi",
            authors=[Author(name="Iain M. Banks")],
            publisher=Publisher(name="Macmillan"),
        )

        self.assertEqual(
"""<Book fiction="True">
<title>Consider Phlebas &amp; Other stories</title>
<isbn>0-333-45430-8</isbn>
<num_pages>471</num_pages>
<rrp>19.5</rrp>
<genre>sci-fi</genre>
<authors>
<Author>
<name>Iain M. Banks</name>
</Author>
</authors>
<Publisher>
<name>Macmillan</name>
</Publisher>
</Book>
""", xml_codec.dumps(book, line_ending='\n'))



# -*- coding: utf-8 -*-
import unittest
import warnings
from odin.compatibility import deprecated


class DeprecatedTestCase(unittest.TestCase):
    def test_function_deprecation_warning(self):
        @deprecated("No longer used.", category=UserWarning)
        def deprecated_function():
            pass

        with warnings.catch_warnings(record=True) as warning_log:
            deprecated_function()

        # Compare the message values
        self.assertListEqual([
            str(m.message) for m in
            sorted(warning_log, key=lambda l: str(l.message))
        ], [
            'deprecated_function is deprecated and scheduled for removal. No longer used.',
        ])

    def test_class_deprecation_warning(self):
        @deprecated("No longer used.", category=UserWarning)
        class DeprecatedClass(object):
            pass

        with warnings.catch_warnings(record=True) as warning_log:
            DeprecatedClass()

        # Compare the message values
        self.assertListEqual([
            str(m.message) for m in
            sorted(warning_log, key=lambda l: str(l.message))
        ], [
            'DeprecatedClass is deprecated and scheduled for removal. No longer used.',
        ])




# -*- coding: utf-8 -*-
"""
Query language for filtering::

    field == 42 AND field_2 == 'abc cde' OR field_3 = Null

"""
from odin import filtering
try:
    from ply import lex
except ImportError:
    raise ImportError("The ply package is not installed, please install this library to use filter_query syntax.")


import warnings
warnings.warn(message="This is a pre-release feature that is very much a work in progress.")


keywords = (
    'DATE', 'DATETIME'
)

tokens = (
    'EQ', 'NWQ', 'LT', 'LTE', 'GT', 'GTE', 'AND', 'OR',
    'STRING', 'INTEGER', 'FLOAT', 'REFERENCE',
    'LPAREN', 'RPAREN', 'NONE'
)


t_STRING = r'\".+\"'


def t_INTEGER(t):
    r'\d+'
    t.value = int(t.value)
    return t


def t_FLOAT(t):
    r'\d+\.\d+'
    t.value = float(t.value)
    return t


FILTER_OPERATOR_MAP = {
    "=": filtering.Equal, "==": filtering.Equal, "eq": filtering.Equal,
    "!=": filtering.NotEqual, "<>": filtering.NotEqual, "neq": filtering.NotEqual,
    "<":  filtering.LessThan, "lt": filtering.LessThan,
    "<=": filtering.LessThanOrEqual, "lte": filtering.LessThanOrEqual,
    ">":  filtering.GreaterThan, "gt": filtering.GreaterThan,
    ">=": filtering.GreaterThanOrEqual, "gte": filtering.GreaterThanOrEqual,
    "in": "In",
    "is": "Is"
}


def _tokenize_string(query_str):
    tokens = []
    in_string = False
    for c in query_str:
        if c == ' ':
            pass
    return tokens


def parse(query):
    tokens = _tokenize_string(query)
